import numpy as np

import matplotlib.pyplot as plt

from math import *

##########
##  Q1  ##
##########

def Euler_n_corr(n,N,y0):
    ## Cette fonction prend en entrée N, la puissance à laquelle est la fonction y
    ## dans l'équation différentielle y' = y^N qu'on cherche à résoudre
    ## sur l'intervalle [0,1]
    ## n est le nombre de points désirés et y0 la valeur de y(0) 
    # Calcul du pas de la discrétisation du temps entre 0 et 1 avec n points
    pas=1/(n-1)
    # on construit l'approximation de y avec la méthode d'Euler
    y=[y0]
    for i in range(n-1) :
        y.append(y[-1]+pas*(y[-1])**N)    
    return(y)

##########
##  Q2  ##
##########
    
def Comparaison(n): 
   ## Cette fonction compare la valeur exacte et la valeur approchée
   ## de la solution de l'équation différentielle y' = y^2 sur l'intervalle [0,1]
   ## avec y(0) = 1/2
    temps_discret=np.linspace(0,1,n)    

    y_approx=Euler_n_corr(n,2,0.5)  

    y_exact=1/(2-temps_discret)
    plt.plot(temps_discret,y_approx,'-r',temps_discret,y_exact,'-b') 


##########
##  Q5  ##
##########
    
def Comparaison2(n,N,y0): 
   ## Cette fonction est une généralisation de la fonction précédente. 
   ## Pour toute équation différentielle de la forme y' = y^N, elle trace
   ## la solution exacte et la solution approchée par la méthode d'Euler
   ## sur l'intervalle [0,1] avec y0 = y(0) et n le nombre de points désirés
    temps_discret=np.linspace(0,1,n)    

    y_approx=Euler_n_corr(n,N,y0)  

    y_exact=(y0**(1-N)+(1-N)*temps_discret)**(1/(1-N))
    plt.plot(temps_discret,y_approx,'-r',temps_discret,y_exact,'-b') 




##########
##  Q6  ##
##########
   
def Comparaison3_corr(n,y0):  
   ## Pour les équations différentielles de la forme y' = y^N avec N prenant 
   ## les valeurs 2,3,4,5,6,7; cette fonction trace
   ## les solutions exactes et les solutions approchées par la méthode d'Euler
   ## sur l'intervalle [0,1] avec y0 = y(0) et n le nombre de points désirés
    temps_discret=np.linspace(0,1,n)    
    for N in range(2,7) :
        y_approx=Euler_n_corr(n,N,y0) 
        y_exact=(y0**(1-N)+(1-N)*temps_discret)**(1/(1-N))
        plt.plot(temps_discret,y_approx,'-r',temps_discret,y_exact,'-b') 
        
        
######
# Q8 #
######

def y(t):
    #### y(t) est la fonction qui donne l'altitude de la balle en fonction 
    ### du temps. Elle prend comme conditions initiales une vitesse de 
    ## 4 m.s^-1, un angle de 45° et une hauteur de 0,5 m
    return(-4.9*t**2+4*sin(radians(45))*t+0.5)
    
def dy(t):
    ## dy(t) est la fonction qui donne la dérivée de l'altitude en fonction 
    ## du temps (càd la vitesse sur l'axe y)
    return(-9.8*t+4*sin(radians(45)))


def trajectoire_corr(n):
    ## cette fonction trace l'altitude en fonction du temps avec n points
    temps = np.linspace(0,1,n)
    altitude = y(temps)
    
    plt.xlabel("temps en s")
    plt.ylabel("altitude en m")
    plt.plot(temps, altitude, "-b")
    plt.legend(["courbe de l'altitude en fonction du temps"])
    plt.show()
    

##########
##  Q9 ##
##########
    
def temps_chute(y,dy,a,epsilon):
    ## Cette fonction prend en argument la fonction y qui donne l'altitude
    ## en fonction du temps, sa dérivée dy, un point a choisi dans l'intervalle
    ## [0,1] et la précision epsilon souhaitée.
    ## Elle renvoie le temps pour lequel la balle passe sous la table de TP 
    ## (le temps pour lequel l'altitude devient négative) avec une précision 
    ## epsilon sur le résultat
    b = a - y(a)/dy(a)
    while abs(b-a)>=2*epsilon:
        a = b
        b = b -y(b)/dy(b)
    return(b)
    

##########
##  Q11 ##
##########    

def dichotomie_maximum(dy,a,b,epsilon):
    ## L'altitude maximale correspond au moment où la vitesse de la balle
    ## (càd la dérivée de la position) s'annule. Il suffit donc de chercher
    ## grâce à la méthode de la dichotomie le temps pour lequel la fonction
    ## dy vaut zéro. On se place sur l'intervalle [0,1] et on cherche le 
    ## résultat avec une précision epsilon. b > a
    a_n = a
    b_n = b
    while abs(b_n-a_n)>=2*epsilon:
        c = (b_n+a_n)/2
        if dy(c)*dy(b)<0:
            a_n = c
        else:
            b_n = c
    return(c)
    
##########
##  Q12 ##
##########    

def ddy(t):
    ## ddy(t) est la fonction qui donne la dérivée seconde de l'altitude en fonction 
    ## du temps (càd l'accélération sur l'axe y)
    return(-9.8)

def Newton_maximum(dy,ddy,a,epsilon):
    ## L'altitude maximale correspond au moment où la vitesse de la balle
    ## (càd la dérivée de la position) s'annule. Il suffit donc de chercher
    ## grâce à la méthode de Newton le temps pour lequel la fonction
    ## dy vaut zéro. On se place sur l'intervalle [0,1] et on cherche le 
    ## résultat avec une précision epsilon.
    b = a - dy(a)/ddy(a)
    while abs(b-a)>=2*epsilon:
        a = b
        b = b - dy(b)/ddy(b)
    return(b)

##########
##  Q14 ##
########## 

def dichotomie_sol(y,a,b,epsilon):
    ## La fonction prend en argument une fonction y (la position en fonction du temps)
    ## un intervalle [0,1] ici et une précision epsilon. 
    ## Elle renvoie l'instant pour lequel la balle atteint l'altitude -0,80m
    ## avec une précision epsilon. 
    a_n = a
    b_n = b
    while(b_n-a_n)>=2*epsilon:
        c = (b_n+a_n)/2
        if (y(c)+0.80)*(y(b)+0.80)<0:
            ## Pour pouvoir savoir facilement dans quel demi-intervalle se trouve
            ## la valeur telle que y(t) = -0,80 on décale l'inégalité : 
            ## au lieu de comparer y(b) > ou < -0.80 et y(c) < ou > -0.80 
            ## on compare y(b)+0.80 > ou < à 0 et y(c)+0.80 < ou > à 0 
            a_n = c
        else:
            b_n = c
    return(c)







        
    
    
    
    
    
    
        